@@ -743,7 +743,7 @@ def reduce_df(
         )
 
     if xcats is not None:
-        if not isinstance(xcats, list):
+        if isinstance(xcats, str):
             xcats = [xcats]
 
     if start:
