@@ -350,15 +350,19 @@ def _parse_labels(keys: List[str], labels: Union[List[str], Dict[str, str]], lab
 
     labels_dict = {}
     if labels is not None:
-        assert len(labels) == len(keys), (
-            f"The number of labels provided for the {label_type} must match the "
-            f"number of {label_type}."
-        )
         if isinstance(labels, list):
+            if len(labels) != len(keys):
+                raise ValueError(
+                    f"The number of labels provided for the {label_type} must match the "
+                    f"number of {label_type}."
+                )
             for key, label in zip(keys, labels):
                 labels_dict[key] = label
         elif isinstance(labels, dict):
-            labels_dict = labels
+            labels_dict = labels.copy()
+            missing = [key for key in keys if key not in labels_dict]
+            for key in missing:
+                labels_dict[key] = key
         else:
             raise ValueError(f"The labels parameter for {label_type} must be a list or a dictionary.")
     else:
